/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
*
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.wtp.releng.tools.component.javadoc;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class ClassJavadoc
{
  String name;
  Boolean since;
  List methodJavadocs;

  public String getName()
  {
    return name;
  }

  public void setName(String name)
  {
    this.name = name;
  }

  public boolean isSince()
  {
    if (since == null)
      return true;
    else
      return since.booleanValue();
  }

  public Boolean getSince()
  {
    return since;
  }

  public void setSince(Boolean since)
  {
    this.since = since;
  }

  public void addMethodJavadoc(MethodJavadoc methodJavadoc)
  {
    if (methodJavadocs == null)
      methodJavadocs = new ArrayList();
    methodJavadocs.add(methodJavadoc);
  }

  public List getMethodJavadocs()
  {
    ArrayList copy = new ArrayList();
    if (methodJavadocs != null)
      copy.addAll(methodJavadocs);
    return copy;
  }

  public void setMethodJavadocs(List methodJavadocs)
  {
    this.methodJavadocs = methodJavadocs;
  }

  public int sizeMethodJavadocs()
  {
    return methodJavadocs == null ? 0 : methodJavadocs.size();
  }

  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("<class-javadoc");
    saveAttribute(sb, "name", getName());
    if (getSince() != null)
      saveAttribute(sb, "since", String.valueOf(isSince()));
    sb.append(">");
    for (Iterator it = getMethodJavadocs().iterator(); it.hasNext();)
      sb.append(it.next().toString());
    sb.append("</class-javadoc>");
    return sb.toString();
  }

  protected void saveAttribute(StringBuffer sb, String key, String value)
  {
    if (key != null && value != null)
    {
      sb.append(" ");
      sb.append(key);
      sb.append("=\"");
      sb.append(value);
      sb.append("\"");
    }
  }
}
