/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.wtp.releng.tools.component.javadoc;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.eclipse.core.runtime.Platform;

public class ComponentAPIJavadoc
{
  String name;
  List classJavadocs;
  String location;

  public String getName()
  {
    return name;
  }

  public void setName(String name)
  {
    this.name = name;
  }

  public void addClassJavadoc(ClassJavadoc classJavadoc)
  {
    if (classJavadocs == null)
      classJavadocs = new ArrayList();
    classJavadocs.add(classJavadoc);
  }

  public void addAllClassJavadoc(Collection collection)
  {
    if (classJavadocs == null)
      classJavadocs = new ArrayList();
    classJavadocs.addAll(collection);
  }

  public List getClassJavadocs()
  {
    ArrayList copy = new ArrayList();
    if (classJavadocs != null)
      copy.addAll(classJavadocs);
    return copy;
  }

  public void setClassJavadocs(List classJavadocs)
  {
    this.classJavadocs = classJavadocs;
  }

  public void saveAsHTML(String location) throws TransformerConfigurationException, TransformerException, IOException
  {
    if (location != null)
    {
      this.location = location;
      TransformerFactory factory = TransformerFactory.newInstance();
      Transformer transformer = factory.newTransformer(new StreamSource(Platform.getBundle("org.eclipse.wtp.releng.tools.component.core").getResource("org/eclipse/wtp/releng/tools/component/xsl/component-api-javadoc.xsl").openStream()));
      transformer.transform(new StreamSource(new ByteArrayInputStream(toString().getBytes())), new StreamResult(new FileOutputStream(new File(location))));
    }
  }

  public void save(String location) throws IOException
  {
    if (location != null)
    {
      this.location = location;
      File file = new File(location);
      file.getParentFile().mkdirs();
      FileOutputStream fos = new FileOutputStream(file);
      fos.write(toString().getBytes());
      fos.close();
    }
  }

  public String getLocation()
  {
    return location;
  }

  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("<component-api-javadoc");
    saveAttribute(sb, "name", getName());
    sb.append(">");
    for (Iterator it = getClassJavadocs().iterator(); it.hasNext();)
      sb.append(it.next().toString());
    sb.append("</component-api-javadoc>");
    return sb.toString();
  }

  protected void saveAttribute(StringBuffer sb, String key, String value)
  {
    if (key != null && value != null)
    {
      sb.append(" ");
      sb.append(key);
      sb.append("=\"");
      sb.append(value);
      sb.append("\"");
    }
  }
}
