/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.wtp.releng.tools.component.javadoc;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.Javadoc;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.MethodRef;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.Name;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.TagElement;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.internal.core.util.CharArrayBuffer;
import org.eclipse.wtp.releng.tools.component.IFileLocation;
import org.eclipse.wtp.releng.tools.component.internal.AbstractEmitter;
import org.eclipse.wtp.releng.tools.component.internal.FileLocation;
import org.eclipse.wtp.releng.tools.component.internal.PluginClasspath;
import org.eclipse.wtp.releng.tools.component.model.ComponentXML;
import org.eclipse.wtp.releng.tools.component.model.Package;
import org.eclipse.wtp.releng.tools.component.model.Plugin;
import org.eclipse.wtp.releng.tools.component.model.Type;

public class JavadocCoverageEmitter extends AbstractEmitter implements IPlatformRunnable
{
  private ASTParser parser = ASTParser.newParser(AST.JLS2);

  public Object run(Object arguments)
  {
    try
    {
      String compXMLDir = addTrailingSeperator(System.getProperty("compXMLDir"));
      String srcDir = addTrailingSeperator(System.getProperty("srcDir"));
      String outputDir = addTrailingSeperator(System.getProperty("outputDir"));
      if (compXMLDir != null && srcDir != null && outputDir != null)
      {
        Map compLoc2CompXML = new HashMap();
        harvestComponents(new File(compXMLDir), compLoc2CompXML);
        Map id2Plugin = new HashMap();
        Map id2Fragment = new HashMap();
        harvestPlugins(new File(srcDir), id2Plugin, id2Fragment);
        linkPluginsAndFragments(id2Plugin, id2Fragment);
        JavadocCoverageSummary summary = new JavadocCoverageSummary();
        for (Iterator it = compLoc2CompXML.values().iterator(); it.hasNext();)
          summary.add(genJavadocCoverageXML((ComponentXML)it.next(), id2Plugin, outputDir));
        summary.save(new FileLocation(new File(outputDir + "index-api-javadoc.xml")));
        summary.saveAsHTML("org/eclipse/wtp/releng/tools/component/xsl/component-api-javadoc-summary.xsl", new FileLocation(new File(outputDir + "index-api-javadoc.html")));
      }
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }
    return IPlatformRunnable.EXIT_OK;
  }

  public PluginClasspath getPluginClasspath(IFileLocation fileLocation, boolean validate)
  {
    return super.getPluginClasspath(fileLocation, false);
  }

  private ComponentAPIJavadoc genJavadocCoverageXML(ComponentXML compXML, Map id2Plugin, String outputDir) throws IOException, TransformerConfigurationException, TransformerException
  {
    compXML.load();
    String compName = compXML.getName();
    ComponentAPIJavadoc compAPIJavadoc = new ComponentAPIJavadoc();
    compAPIJavadoc.setName(compName);
    for (Iterator it = compXML.getPlugins().iterator(); it.hasNext();)
    {
      Plugin plugin = (Plugin)it.next();
      Object object = id2Plugin.get(plugin.getId());
      if (object instanceof PluginClasspath)
      {
        PluginClasspath pluginCp = (PluginClasspath)object;
        for (Iterator pkgsIt = compXML.getPackages().iterator(); pkgsIt.hasNext();)
        {
          compAPIJavadoc.addAllClassJavadoc(genJavadocCoverageXML((Package)pkgsIt.next(), addTrailingSeperator(pluginCp.getDotClasspathLocation().getFile().getParentFile().getAbsolutePath()), pluginCp.getSrcPaths()));
        }
      }
    }
    StringBuffer sb = new StringBuffer(outputDir);
    sb.append(compName);
    compAPIJavadoc.save(sb.toString() + "/component-api-javadoc.xml");
    compAPIJavadoc.saveAsHTML(sb.toString() + "/component-api-javadoc.html");
    return compAPIJavadoc;
  }

  private List genJavadocCoverageXML(Package pkg, String baseDir, List srcPaths) throws IOException
  {
    List classJavadocs = new ArrayList();
    String pkgName = pkg.getName();
    String pkgDir = pkgName.replace('.', '/');
    if (pkg.isApi())
    {
      List excludes = new ArrayList();
      List types = pkg.getTypes();
      for (Iterator it = types.iterator(); it.hasNext();)
      {
        Type type = (Type)it.next();
        if (!type.isImplement() && !type.isInstantiate() && !type.isReference() && !type.isSubclass())
          excludes.add(type.getName() + ".java");
      }
      for (Iterator it = srcPaths.iterator(); it.hasNext();)
      {
        StringBuffer sb = new StringBuffer(baseDir);
        sb.append((String)it.next());
        sb.append('/');
        sb.append(pkgDir);
        classJavadocs.addAll(genJavadocCoverageXML(pkgName, new File(sb.toString()), excludes));
      }
    }
    else
    {
      List types = pkg.getTypes();
      for (Iterator it = types.iterator(); it.hasNext();)
      {
        Type type = (Type)it.next();
        if (type.isImplement() || type.isInstantiate() || type.isReference() || type.isSubclass())
        {
          for (Iterator srcIt = srcPaths.iterator(); srcIt.hasNext();)
          {
            StringBuffer sb = new StringBuffer(baseDir);
            sb.append((String)srcIt.next());
            sb.append('/');
            sb.append(pkgDir);
            sb.append('/');
            sb.append(type.getName());
            sb.append(".java");
            classJavadocs.addAll(genJavadocCoverageXML(pkgName, new File(sb.toString()), null));
          }
        }
      }
    }
    return classJavadocs;
  }

  private List genJavadocCoverageXML(String pkg, File file, List excludes) throws IOException
  {
    List classJavadocs = new ArrayList();
    if (file.exists())
    {
      if (file.isDirectory())
      {
        File[] children = file.listFiles();
        for (int i = 0; i < children.length; i++)
          if (children[i].isFile())
            classJavadocs.addAll(genJavadocCoverageXML(pkg, children[i], excludes));
      }
      else if (file.getName().endsWith(".java"))
      {
        if (excludes == null || !excludes.contains(file.getName()))
        {
          FileReader fr = new FileReader(file);
          CharArrayBuffer cab = new CharArrayBuffer();
          char[] c = new char[2048];
          for (int read = fr.read(c); read != -1; read = fr.read(c))
            cab.append(c, 0, read);
          fr.close();
          parser.setSource(cab.getContents());
          ASTNode node = parser.createAST(null);
          JavadocVisitor visitor = new JavadocVisitor(pkg);
          node.accept(visitor);
          ClassJavadoc classJavadoc = visitor.getClassJavadoc();
          if (classJavadoc != null)
            classJavadocs.add(classJavadoc);
        }
      }
    }
    return classJavadocs;
  }
  private class JavadocVisitor extends ASTVisitor
  {
    private String pkg;
    private String className;
    private ClassJavadoc classJavadoc;
    boolean classSince;

    public JavadocVisitor(String pkg)
    {
      this.pkg = pkg;
      classSince = false;
    }

    public boolean visit(TypeDeclaration node)
    {
      className = node.getName().getFullyQualifiedName();
      if (!isBit(node.getModifiers(), Modifier.PRIVATE))
      {
        Javadoc javadoc = node.getJavadoc();
        if (javadoc != null)
        {
          List tags = javadoc.tags();
          for (Iterator it = tags.iterator(); it.hasNext();)
          {
            TagElement tag = (TagElement)it.next();
            String tagName = tag.getTagName();
            if (tagName != null && tagName.equals(TagElement.TAG_SINCE))
            {
              classSince = true;
            }
          }
        }
        if (!classSince)
        {
          ClassJavadoc classJavadoc = getClassJavadoc(true);
          classJavadoc.setSince(Boolean.FALSE);
        }
      }
      return true;
    }

    public boolean visit(MethodDeclaration node)
    {
      if (!isBit(node.getModifiers(), Modifier.PRIVATE))
      {
        String methodName = node.getName().getFullyQualifiedName();
        boolean methodSince = classSince;
        boolean hasReturn = false;
        org.eclipse.jdt.core.dom.Type returnType = node.getReturnType();
        if (returnType instanceof PrimitiveType)
          if (((PrimitiveType)returnType).getPrimitiveTypeCode() == PrimitiveType.VOID)
            hasReturn = true;
        List paramList = new ArrayList();
        List params = node.parameters();
        for (Iterator it = params.iterator(); it.hasNext();)
        {
          SingleVariableDeclaration param = (SingleVariableDeclaration)it.next();
          SimpleName simpleName = param.getName();
          paramList.add(simpleName.getIdentifier());
        }
        List throwList = new ArrayList();
        List thrownExceptions = node.thrownExceptions();
        for (Iterator it = thrownExceptions.iterator(); it.hasNext();)
        {
          Name name = (Name)it.next();
          throwList.add(name.getFullyQualifiedName());
        }
        Javadoc javadoc = node.getJavadoc();
        if (javadoc != null)
        {
          List tags = javadoc.tags();
          for (Iterator it = tags.iterator(); it.hasNext();)
          {
            TagElement tag = (TagElement)it.next();
            String tagName = tag.getTagName();
            if (tagName != null)
            {
              if (tagName.equals(TagElement.TAG_SINCE))
                methodSince = true;
              else if (tagName.equals(TagElement.TAG_RETURN))
                hasReturn = tag.fragments().size() > 0;
              else if (tagName.equals(TagElement.TAG_PARAM))
              {
                List fragments = tag.fragments();
                if (fragments.size() > 1)
                {
                  Object fragment = fragments.get(0);
                  if (fragment instanceof SimpleName)
                  {
                    paramList.remove(((SimpleName)fragment).getIdentifier());
                  }
                }
              }
              else if (tagName.equals(TagElement.TAG_THROWS))
              {
                List fragments = tag.fragments();
                if (fragments.size() > 1)
                {
                  Object fragment = fragments.get(0);
                  if (fragment instanceof SimpleName)
                  {
                    throwList.remove(((SimpleName)fragment).getIdentifier());
                  }
                }
              }
              else if (tagName.equals(TagElement.TAG_SEE))
              {
                List fragments = tag.fragments();
                if (fragments.size() > 0)
                {
                  Object fragment = fragments.get(0);
                  if (fragment instanceof MethodRef)
                  {
                    MethodRef methodRef = (MethodRef)fragment;
                    if (methodName.equals(methodRef.getName().getFullyQualifiedName()))
                    {
                      return true;
                    }
                  }
                }
              }
            }
          }
        }
        if (!methodSince || !hasReturn || paramList.size() > 0 || throwList.size() > 0)
        {
          MethodJavadoc methodJavadoc = new MethodJavadoc();
          methodJavadoc.setName(methodName);
          if (!methodSince)
          {
            methodJavadoc.setSince(Boolean.FALSE);
          }
          if (!hasReturn)
          {
            methodJavadoc.setReturnType(Boolean.FALSE);
          }
          if (paramList.size() > 0)
          {
            for (Iterator it = paramList.iterator(); it.hasNext();)
            {
              methodJavadoc.addParam((String)it.next());
            }
          }
          if (throwList.size() > 0)
          {
            for (Iterator it = throwList.iterator(); it.hasNext();)
            {
              methodJavadoc.addThrow((String)it.next());
            }
          }
          getClassJavadoc(true).addMethodJavadoc(methodJavadoc);
        }
      }
      return true;
    }

    public ClassJavadoc getClassJavadoc()
    {
      return getClassJavadoc(false);
    }

    public ClassJavadoc getClassJavadoc(boolean create)
    {
      if (create && classJavadoc == null)
      {
        classJavadoc = new ClassJavadoc();
        StringBuffer sb = new StringBuffer(pkg);
        sb.append('.');
        sb.append(className);
        classJavadoc.setName(sb.toString());
      }
      return classJavadoc;
    }

    private boolean isBit(int flag, int bit)
    {
      return ((flag & bit) == bit);
    }
  }
}
