/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.wtp.releng.tools.component.javadoc;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.eclipse.core.runtime.Platform;
import org.eclipse.wtp.releng.tools.component.ILocation;
import org.eclipse.wtp.releng.tools.component.internal.ComponentEntry;
import org.eclipse.wtp.releng.tools.component.internal.ComponentSummary;

public class JavadocCoverageSummary extends ComponentSummary
{
  private static final String ROOT_TAG_NAME = "component-api-javadoc-summary";

  public void add(ComponentAPIJavadoc compAPIJavadoc)
  {
    JavadocCoverageEntry entry = new JavadocCoverageEntry();
    int count = 0;
    entry.setCompName(compAPIJavadoc.getName());
    for (Iterator it = compAPIJavadoc.getClassJavadocs().iterator(); it.hasNext();)
    {
      count++;
      count += ((ClassJavadoc)it.next()).sizeMethodJavadocs();
    }
    entry.setCount(count);
    entry.setRef(compAPIJavadoc.getLocation());
    add(entry);
  }

  protected void saveAsHTML(ILocation html, String xsl, String rootTagName) throws TransformerConfigurationException, TransformerException, IOException
  {
    TransformerFactory factory = TransformerFactory.newInstance();
    Transformer transformer = factory.newTransformer(new StreamSource(Platform.getBundle("org.eclipse.wtp.releng.tools.component.core").getResource(xsl).openStream()));
    transformer.transform(new StreamSource(new ByteArrayInputStream(getBytes(html, rootTagName))), new StreamResult(new FileOutputStream(new File(html.getAbsolutePath()))));
  }

  public void saveAsHTML(String xsl, ILocation html) throws TransformerConfigurationException, TransformerException, IOException
  {
    saveAsHTML(html, xsl, ROOT_TAG_NAME);
  }

  public void save(ILocation location) throws IOException
  {
    save(location, ROOT_TAG_NAME);
  }
  private class JavadocCoverageEntry extends ComponentEntry
  {
    private static final String ATTR_COUNT = "count";
    private int count;

    public JavadocCoverageEntry()
    {
      count = 0;
    }

    public void save(StringBuffer sb)
    {
      sb.append("<component-api-javadoc ");
      saveAttribute(sb, ATTR_NAME, getCompName());
      saveAttribute(sb, ATTR_COUNT, String.valueOf(count));
      saveAttribute(sb, ATTR_REF, getRef());
      sb.append("/>");
    }

    /**
     * @return Returns the count.
     */
    public int getCount()
    {
      return count;
    }

    /**
     * @param apiCount
     *          The count to set.
     */
    public void setCount(int count)
    {
      this.count = count;
    }
  }
}
