/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
*
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.wtp.releng.tools.component.javadoc;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class MethodJavadoc
{
  String name;
  Boolean since;
  Boolean returnType;
  List paramList;
  List throwList;

  public void addParam(String param)
  {
    if (paramList == null)
      paramList = new ArrayList();
    paramList.add(param);
  }

  public List getParamList()
  {
    List copy = new ArrayList();
    if (paramList != null)
      copy.addAll(paramList);
    return copy;
  }

  public void setParamList(List paramList)
  {
    this.paramList = paramList;
  }

  public boolean isReturnType()
  {
    if (returnType == null)
      return true;
    else
      return returnType.booleanValue();
  }

  public Boolean getReturnType()
  {
    return returnType;
  }

  public void setReturnType(Boolean returnType)
  {
    this.returnType = returnType;
  }

  public void addThrow(String throwType)
  {
    if (throwList == null)
      throwList = new ArrayList();
    throwList.add(throwType);
  }

  public List getThrowList()
  {
    List copy = new ArrayList();
    if (throwList != null)
      copy.addAll(throwList);
    return copy;
  }

  public void setThrowList(List throwList)
  {
    this.throwList = throwList;
  }

  public String getName()
  {
    return name;
  }

  public void setName(String name)
  {
    this.name = name;
  }

  public boolean isSince()
  {
    if (since == null)
      return true;
    else
      return since.booleanValue();
  }

  public Boolean getSince()
  {
    return since;
  }

  public void setSince(Boolean since)
  {
    this.since = since;
  }

  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("<method-javadoc");
    saveAttribute(sb, "name", getName());
    if (getSince() != null)
      saveAttribute(sb, "since", String.valueOf(isSince()));
    if (getReturnType() != null)
      saveAttribute(sb, "return", String.valueOf(isReturnType()));
    sb.append(">");
    for (Iterator it = getParamList().iterator(); it.hasNext();)
    {
      sb.append("<param");
      saveAttribute(sb, "name", (String)it.next());
      sb.append("/>");
    }
    for (Iterator it = getThrowList().iterator(); it.hasNext();)
    {
      sb.append("<throw");
      saveAttribute(sb, "name", (String)it.next());
      sb.append("/>");
    }
    sb.append("</method-javadoc>");
    return sb.toString();
  }

  protected void saveAttribute(StringBuffer sb, String key, String value)
  {
    if (key != null && value != null)
    {
      sb.append(" ");
      sb.append(key);
      sb.append("=\"");
      sb.append(value);
      sb.append("\"");
    }
  }

  protected void saveAttribute(StringBuffer sb, String key, List values, String delimiter)
  {
    if (key != null && values != null && values.size() > 0)
    {
      sb.append(" ");
      sb.append(key);
      sb.append("=\"");
      for (Iterator it = values.iterator(); it.hasNext();)
      {
        sb.append(it.next().toString());
        if (it.hasNext())
          sb.append(delimiter);
      }
      sb.append("\"");
    }
  }
}
